@@ -11,13 +11,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import itertools
 import os
 import platform
 import shutil
 import sys
 from datetime import datetime
 from pathlib import Path
-from typing import Optional
+from typing import Optional, List
 
 import click
 
@@ -29,16 +30,23 @@ from lean.models.errors import MoreInfoError
 class LeanCommand(click.Command):
     """A click.Command wrapper with some Lean CLI customization."""
 
-    def __init__(self, requires_lean_config: bool = False, requires_docker: bool = False, *args, **kwargs):
+    def __init__(self,
+                 requires_lean_config: bool = False,
+                 requires_docker: bool = False,
+                 allow_unknown_options: bool = False,
+                 *args,
+                 **kwargs):
         """Creates a new LeanCommand instance.
 
         :param requires_lean_config: True if this command requires a Lean config, False if not
         :param requires_docker: True if this command uses Docker, False if not
+        :param allow_unknown_options: True if unknown options are allowed, False if not
         :param args: the args that are passed on to the click.Command constructor
         :param kwargs: the kwargs that are passed on to the click.Command constructor
         """
         self._requires_lean_config = requires_lean_config
         self._requires_docker = requires_docker
+        self._allow_unknown_options = allow_unknown_options
 
         super().__init__(*args, **kwargs)
 
@@ -46,7 +54,11 @@ class LeanCommand(click.Command):
         # max_content_width defaults to 80, which we increase to 120 to improve readability on wide terminals
         self.context_settings["max_content_width"] = 120
 
-    def invoke(self, ctx):
+        # Don't fail if unknown options are passed in when they're allowed
+        self.context_settings["ignore_unknown_options"] = allow_unknown_options
+        self.context_settings["allow_extra_args"] = allow_unknown_options
+
+    def invoke(self, ctx: click.Context):
         if self._requires_lean_config:
             try:
                 # This method will throw if the directory cannot be found
@@ -77,6 +89,24 @@ class LeanCommand(click.Command):
                 args = ["sudo", "--preserve-env=HOME", sys.executable, *sys.argv]
                 os.execlp(args[0], *args)
 
+        if self._allow_unknown_options:
+            # Unknown options are passed to ctx.args and need to be parsed manually
+            # We parse them to ctx.params so they're available like normal options
+            # Because of this all commands with allow_unknown_options=True must have a **kwargs argument
+            arguments = list(itertools.chain(*[arg.split("=") for arg in ctx.args]))
+
+            skip_next = False
+            for index in range(len(arguments) - 1):
+                if skip_next:
+                    skip_next = False
+                    continue
+
+                if arguments[index].startswith("--"):
+                    option = arguments[index].replace("--", "")
+                    value = arguments[index + 1]
+                    ctx.params[option] = value
+                    skip_next = True
+
         update_manager = container.update_manager()
         update_manager.show_announcements()
 
@@ -86,7 +116,7 @@ class LeanCommand(click.Command):
 
         return result
 
-    def get_params(self, ctx):
+    def get_params(self, ctx: click.Context):
         params = super().get_params(ctx)
 
         # Add --lean-config option if the command requires a Lean config
@@ -119,7 +149,7 @@ class LeanCommand(click.Command):
         """Parses the --verbose option."""
         if value:
             logger = container.logger()
-            logger.enable_debug_logging()
+            logger.debug_logging_enabled = True
 
 
 class PathParameter(click.ParamType):
@@ -180,3 +210,38 @@ class DateParameter(click.ParamType):
                 pass
 
         self.fail(f"'{value}' does not match the yyyyMMdd format.", param, ctx)
+
+
+def ensure_options(ctx: click.Context, options: List[str]) -> None:
+    """Ensures certain options have values, raises an error if not.
+
+    :param ctx: the click context of the invocation
+    :param options: the Python names of the options that must have values
+    """
+    missing_options = []
+    for key, value in ctx.params.items():
+        has_value = value is not None
+
+        if isinstance(value, tuple) and len(value) == 0:
+            has_value = False
+
+        if not has_value and key in options:
+            missing_options.append(key)
+
+    if len(missing_options) == 0:
+        return
+
+    missing_options = sorted(missing_options, key=lambda param: options.index(param))
+    help_records = []
+
+    for name in missing_options:
+        option = next(param for param in ctx.command.params if param.name == name)
+        help_records.append(option.get_help_record(ctx))
+
+    help_formatter = click.HelpFormatter(max_width=120)
+    help_formatter.write_dl(help_records)
+
+    raise RuntimeError(f"""
+You are missing the following option{"s" if len(missing_options) > 1 else ""}:
+{''.join(help_formatter.buffer)}
+    """.strip())
