@@ -12,6 +12,7 @@
 # limitations under the License.
 
 import json
+import os
 from datetime import datetime
 from pathlib import Path
 from typing import Optional
@@ -23,53 +24,111 @@ from lean.constants import DEFAULT_ENGINE_IMAGE
 from lean.container import container
 from lean.models.api import QCMinimalOrganization
 from lean.models.config import DebuggingMethod
+from lean.models.data_providers import all_data_providers
+from lean.models.data_providers.quantconnect import QuantConnectDataProvider
 from lean.models.logger import Option
 
 
-# The _migrate_dotnet_5_* methods automatically update launch configurations for a given debugging method.
+# The _migrate_* methods automatically update launch configurations for a given debugging method.
 #
-# In the update to .NET 5, debugging changed considerably.
-# This lead to some required changes in the launch configurations users use to start debugging.
-# Projects which are created after the update to .NET 5 have the correct configuration already,
-# but projects created before that need changes to their launch configurations.
+# Occasionally we make changes which require updated launch configurations.
+# Projects which are created after these update have the correct configuration already,
+# but projects created before that need changes.
 #
-# These methods checks if the project have outdated configurations, and if so, update them to keep it working.
-# These methods will only be useful for the first few weeks after the update to .NET 5 and will be removed afterwards.
+# These methods checks if the project has outdated configurations, and if so, update them to keep it working.
 
-def _migrate_dotnet_5_python_pycharm(project_dir: Path) -> None:
+def _migrate_python_pycharm(project_dir: Path) -> None:
     workspace_xml_path = project_dir / ".idea" / "workspace.xml"
     if not workspace_xml_path.is_file():
         return
 
-    current_content = workspace_xml_path.read_text(encoding="utf-8")
-    if 'remote-root="/Lean/Launcher/bin/Debug"' not in current_content:
+    xml_manager = container.xml_manager()
+    current_content = xml_manager.parse(workspace_xml_path.read_text(encoding="utf-8"))
+
+    config = current_content.find('.//configuration[@name="Debug with Lean CLI"]')
+    if config is None:
         return
 
-    new_content = current_content.replace('remote-root="/Lean/Launcher/bin/Debug"', 'remote-root="/LeanCLI"')
-    workspace_xml_path.write_text(new_content, encoding="utf-8")
+    path_mappings = config.find('.//PathMappingSettings/option[@name="pathMappings"]/list')
+    if path_mappings is None:
+        return
 
-    logger = container.logger()
-    logger.warn("Your run configuration has been updated to work with the latest version of LEAN")
-    logger.warn("Please restart the debugger in PyCharm and run this command again")
+    made_changes = False
+    has_library_mapping = False
 
-    raise click.Abort()
+    library_dir = container.lean_config_manager().get_cli_root_directory() / "Library"
+    if library_dir.is_dir():
+        library_dir = f"$PROJECT_DIR$/{os.path.relpath(library_dir, project_dir)}".replace("\\", "/")
+    else:
+        library_dir = None
+
+    for mapping in path_mappings.findall(".//mapping"):
+        if mapping.get("local-root") == "$PROJECT_DIR$" and mapping.get("remote-root") == "/Lean/Launcher/bin/Debug":
+            mapping.set("remote-root", "/LeanCLI")
+            made_changes = True
+
+        if library_dir is not None \
+            and mapping.get("local-root") == library_dir \
+            and mapping.get("remote-root") == "/Library":
+            has_library_mapping = True
+
+    if library_dir is not None and not has_library_mapping:
+        library_mapping = xml_manager.parse("<mapping/>")
+        library_mapping.set("local-root", library_dir)
+        library_mapping.set("remote-root", "/Library")
+        path_mappings.append(library_mapping)
+        made_changes = True
+
+    if made_changes:
+        workspace_xml_path.write_text(xml_manager.to_string(current_content), encoding="utf-8")
+
+        logger = container.logger()
+        logger.warn("Your run configuration has been updated to work with the latest version of LEAN")
+        logger.warn("Please restart the debugger in PyCharm and run this command again")
+
+        raise click.Abort()
 
 
-def _migrate_dotnet_5_python_vscode(project_dir: Path) -> None:
+def _migrate_python_vscode(project_dir: Path) -> None:
     launch_json_path = project_dir / ".vscode" / "launch.json"
     if not launch_json_path.is_file():
         return
 
-    current_content = launch_json_path.read_text(encoding="utf-8")
-    if '"remoteRoot": "/Lean/Launcher/bin/Debug"' not in current_content:
+    current_content = json.loads(launch_json_path.read_text(encoding="utf-8"))
+    if "configurations" not in current_content or not isinstance(current_content["configurations"], list):
         return
 
-    new_content = current_content.replace('"remoteRoot": "/Lean/Launcher/bin/Debug"',
-                                          '"remoteRoot": "/LeanCLI"')
-    launch_json_path.write_text(new_content, encoding="utf-8")
+    config = next((c for c in current_content["configurations"] if c["name"] == "Debug with Lean CLI"), None)
+    if config is None:
+        return
+
+    made_changes = False
+    has_library_mapping = False
+
+    library_dir = container.lean_config_manager().get_cli_root_directory() / "Library"
+    if not library_dir.is_dir():
+        library_dir = None
+
+    for mapping in config["pathMappings"]:
+        if mapping["localRoot"] == "${workspaceFolder}" and mapping["remoteRoot"] == "/Lean/Launcher/bin/Debug":
+            mapping["remoteRoot"] = "/LeanCLI"
+            made_changes = True
+
+        if library_dir is not None and mapping["localRoot"] == str(library_dir) and mapping["remoteRoot"] == "/Library":
+            has_library_mapping = True
+
+    if library_dir is not None and not has_library_mapping:
+        config["pathMappings"].append({
+            "localRoot": str(library_dir),
+            "remoteRoot": "/Library"
+        })
+        made_changes = True
+
+    if made_changes:
+        launch_json_path.write_text(json.dumps(current_content, indent=4), encoding="utf-8")
 
 
-def _migrate_dotnet_5_csharp_rider(project_dir: Path) -> None:
+def _migrate_csharp_rider(project_dir: Path) -> None:
     made_changes = False
     xml_manager = container.xml_manager()
 
@@ -105,7 +164,7 @@ def _migrate_dotnet_5_csharp_rider(project_dir: Path) -> None:
         raise click.Abort()
 
 
-def _migrate_dotnet_5_csharp_vscode(project_dir: Path) -> None:
+def _migrate_csharp_vscode(project_dir: Path) -> None:
     launch_json_path = project_dir / ".vscode" / "launch.json"
     if not launch_json_path.is_file():
         return
@@ -142,6 +201,29 @@ def _migrate_dotnet_5_csharp_vscode(project_dir: Path) -> None:
     launch_json_path.write_text(json.dumps(current_content, indent=4), encoding="utf-8")
 
 
+def _migrate_csharp_csproj(project_dir: Path) -> None:
+    csproj_path = next((f for f in project_dir.rglob("*.csproj")), None)
+    if csproj_path is None:
+        return
+
+    xml_manager = container.xml_manager()
+
+    current_content = xml_manager.parse(csproj_path.read_text(encoding="utf-8"))
+    if current_content.find(".//PropertyGroup/DefaultItemExcludes") is not None:
+        return
+
+    property_group = current_content.find(".//PropertyGroup")
+    if property_group is None:
+        property_group = xml_manager.parse("<PropertyGroup/>")
+        current_content.append(property_group)
+
+    default_item_excludes = xml_manager.parse(
+        "<DefaultItemExcludes>$(DefaultItemExcludes);backtests/*/code/**;live/*/code/**;optimizations/*/code/**</DefaultItemExcludes>")
+    property_group.append(default_item_excludes)
+
+    csproj_path.write_text(xml_manager.to_string(current_content), encoding="utf-8")
+
+
 def _select_organization() -> QCMinimalOrganization:
     """Asks the user for the organization that should be charged when downloading data.
 
@@ -164,13 +246,20 @@ def _select_organization() -> QCMinimalOrganization:
 @click.option("--debug",
               type=click.Choice(["pycharm", "ptvsd", "vsdbg", "rider"], case_sensitive=False),
               help="Enable a certain debugging method (see --help for more information)")
+@click.option("--data-provider",
+              type=click.Choice([dp.get_name() for dp in all_data_providers], case_sensitive=False),
+              help="Update the Lean configuration file to retrieve data from the given provider")
 @click.option("--download-data",
               is_flag=True,
               default=False,
-              help="Update the Lean configuration file to download data from the QuantConnect API")
+              help="Update the Lean configuration file to download data from the QuantConnect API, alias for --data-provider QuantConnect")
 @click.option("--data-purchase-limit",
               type=int,
-              help="The maximum amount of QCC to spend on downloading data during this backtest")
+              help="The maximum amount of QCC to spend on downloading data during the backtest when using QuantConnect as data provider")
+@click.option("--release",
+              is_flag=True,
+              default=False,
+              help="Compile C# projects in release configuration instead of debug")
 @click.option("--image",
               type=str,
               help=f"The LEAN engine image to use (defaults to {DEFAULT_ENGINE_IMAGE})")
@@ -181,8 +270,10 @@ def _select_organization() -> QCMinimalOrganization:
 def backtest(project: Path,
              output: Optional[Path],
              debug: Optional[str],
+             data_provider: Optional[str],
              download_data: bool,
              data_purchase_limit: Optional[int],
+             release: bool,
              image: Optional[str],
              update: bool) -> None:
     """Backtest a project locally using Docker.
@@ -206,34 +297,35 @@ def backtest(project: Path,
     if output is None:
         output = algorithm_file.parent / "backtests" / datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
 
+    project_manager.copy_code(algorithm_file.parent, output / "code")
+
     debugging_method = None
     if debug == "pycharm":
         debugging_method = DebuggingMethod.PyCharm
-        _migrate_dotnet_5_python_pycharm(algorithm_file.parent)
+        _migrate_python_pycharm(algorithm_file.parent)
     elif debug == "ptvsd":
         debugging_method = DebuggingMethod.PTVSD
-        _migrate_dotnet_5_python_vscode(algorithm_file.parent)
+        _migrate_python_vscode(algorithm_file.parent)
     elif debug == "vsdbg":
         debugging_method = DebuggingMethod.VSDBG
-        _migrate_dotnet_5_csharp_vscode(algorithm_file.parent)
+        _migrate_csharp_vscode(algorithm_file.parent)
     elif debug == "rider":
         debugging_method = DebuggingMethod.Rider
-        _migrate_dotnet_5_csharp_rider(algorithm_file.parent)
+        _migrate_csharp_rider(algorithm_file.parent)
+
+    if algorithm_file.name.endswith(".cs"):
+        _migrate_csharp_csproj(algorithm_file.parent)
+
+    lean_config = lean_config_manager.get_complete_lean_config("backtesting", algorithm_file, debugging_method)
 
     if download_data:
-        organization = _select_organization()
-        lean_config_manager.set_property("job-organization-id", organization.id)
-        lean_config_manager.set_property("data-provider", "QuantConnect.Lean.Engine.DataFeeds.ApiDataProvider")
-        lean_config_manager.set_property("map-file-provider", "QuantConnect.Data.Auxiliary.LocalZipMapFileProvider")
-        lean_config_manager.set_property("factor-file-provider",
-                                         "QuantConnect.Data.Auxiliary.LocalZipFactorFileProvider")
+        data_provider = QuantConnectDataProvider.get_name()
 
-    if data_purchase_limit is not None:
-        config = lean_config_manager.get_lean_config()
-        if config.get("data-provider", None) != "QuantConnect.Lean.Engine.DataFeeds.ApiDataProvider":
-            container.logger().warn(
-                "--data-purchase-limit is ignored because the data provider is not set to download from the API, use --download-data to set that up")
-            data_purchase_limit = None
+    if data_provider is not None:
+        data_provider = next(dp for dp in all_data_providers if dp.get_name() == data_provider)
+        data_provider.build(lean_config, container.logger()).configure(lean_config, "backtesting")
+
+    lean_config_manager.configure_data_purchase_limit(lean_config, data_purchase_limit)
 
     cli_config_manager = container.cli_config_manager()
     engine_image = cli_config_manager.get_engine_image(image)
@@ -243,13 +335,8 @@ def backtest(project: Path,
     if update or not docker_manager.supports_dotnet_5(engine_image):
         docker_manager.pull_image(engine_image)
 
-    lean_config = lean_config_manager.get_complete_lean_config("backtesting",
-                                                               algorithm_file,
-                                                               debugging_method,
-                                                               data_purchase_limit)
-
     lean_runner = container.lean_runner()
-    lean_runner.run_lean(lean_config, "backtesting", algorithm_file, output, engine_image, debugging_method)
+    lean_runner.run_lean(lean_config, "backtesting", algorithm_file, output, engine_image, debugging_method, release)
 
     if str(engine_image) == DEFAULT_ENGINE_IMAGE and not update:
         update_manager = container.update_manager()
