@@ -19,6 +19,7 @@ import json5
 
 from lean.components.config.cli_config_manager import CLIConfigManager
 from lean.components.config.project_config_manager import ProjectConfigManager
+from lean.components.util.logger import Logger
 from lean.constants import DEFAULT_LEAN_CONFIG_FILE_NAME
 from lean.models.config import DebuggingMethod
 from lean.models.errors import MoreInfoError
@@ -27,12 +28,17 @@ from lean.models.errors import MoreInfoError
 class LeanConfigManager:
     """The LeanConfigManager class contains utilities to work with files containing LEAN engine configuration."""
 
-    def __init__(self, cli_config_manager: CLIConfigManager, project_config_manager: ProjectConfigManager) -> None:
+    def __init__(self,
+                 logger: Logger,
+                 cli_config_manager: CLIConfigManager,
+                 project_config_manager: ProjectConfigManager) -> None:
         """Creates a new LeanConfigManager instance.
 
+        :param logger: the logger to log messages with
         :param cli_config_manager: the CLIConfigManager instance to use when retrieving credentials
         :param project_config_manager: the ProjectConfigManager instance to use when retrieving project parameters
         """
+        self._logger = logger
         self._cli_config_manager = cli_config_manager
         self._project_config_manager = project_config_manager
         self._default_path = None
@@ -86,27 +92,27 @@ class LeanConfigManager:
         config = self.get_lean_config()
         return self.get_cli_root_directory() / config["data-folder"]
 
-    def set_property(self, key: str, value: Any) -> None:
-        """Sets a property in the Lean config file.
+    def set_properties(self, updates: Dict[str, Any]) -> None:
+        """Sets a properties in the Lean config file.
 
-        If the property does not exist yet it is added automatically.
+        If a property does not exist yet it is added automatically.
         Comments in the Lean config file are preserved.
 
-        :param key: the key of the property to update
-        :param value: the new value of the property
+        :param updates: the key -> new value updates to apply to the current config
         """
         config = self.get_lean_config()
 
         config_path = self.get_lean_config_path()
         config_text = config_path.read_text(encoding="utf-8")
 
-        json_value = json5.dumps(value)
+        for key, value in reversed(list(updates.items())):
+            json_value = json5.dumps(value)
 
-        # We can only use regex to set the property because converting the config back to JSON drops all comments
-        if key in config:
-            config_text = re.sub(fr'"{key}":\s*("?[^",]*"?)', f'"{key}": {json_value}', config_text)
-        else:
-            config_text = config_text.replace("{", f'{{\n  "{key}": {json_value},', 1)
+            # We can only use regex to set the property because converting the config back to JSON drops all comments
+            if key in config:
+                config_text = re.sub(fr'"{key}":\s*("?[^",]*"?)', f'"{key}": {json_value}', config_text)
+            else:
+                config_text = config_text.replace("{", f'{{\n  "{key}": {json_value},', 1)
 
         config_path.write_text(config_text, encoding="utf-8")
 
@@ -165,8 +171,7 @@ class LeanConfigManager:
     def get_complete_lean_config(self,
                                  environment: str,
                                  algorithm_file: Path,
-                                 debugging_method: Optional[DebuggingMethod],
-                                 data_purchase_limit: Optional[int]) -> Dict[str, Any]:
+                                 debugging_method: Optional[DebuggingMethod]) -> Dict[str, Any]:
         """Returns a complete Lean config object containing all properties needed for the engine to run.
 
         This retrieves the path of the config, parses the file and adds all properties removed in clean_lean_config().
@@ -174,7 +179,6 @@ class LeanConfigManager:
         :param environment: the environment to set
         :param algorithm_file: the path to the algorithm that will be ran
         :param debugging_method: the debugging method to use, or None to disable debugging
-        :param data_purchase_limit: the data purchase limit in QCC
         """
         config = self.get_lean_config()
 
@@ -190,16 +194,13 @@ class LeanConfigManager:
             config["debugging"] = False
             config["debugging-method"] = "LocalCmdline"
 
-        if data_purchase_limit is not None:
-            config["data-purchase-limit"] = data_purchase_limit
-
         config["job-user-id"] = self._cli_config_manager.user_id.get_value(default="0")
         config["api-access-token"] = self._cli_config_manager.api_token.get_value(default="")
 
         config["ib-host"] = "127.0.0.1"
         config["ib-port"] = "4002"
-        config["ib-tws-dir"] = "/root/Jts"
-        config["ib-version"] = "978"
+        config["ib-tws-dir"] = "/usr/local/ibgateway"
+        config["ib-version"] = "984"
 
         config["iqfeed-host"] = "host.docker.internal"
 
@@ -218,6 +219,24 @@ class LeanConfigManager:
 
         return config
 
+    def configure_data_purchase_limit(self, lean_config: Dict[str, Any], data_purchase_limit: Optional[int]) -> None:
+        """Updates the data purchase limit in the Lean config.
+
+        Logs a warning if the data provider is not configured to download from QuantConnect.
+
+        :param lean_config: the Lean config dict to update
+        :param data_purchase_limit: the data purchase limit provided by the user, or None if no such limit was provided
+        """
+        if data_purchase_limit is None:
+            return
+
+        if lean_config.get("data-provider", None) != "QuantConnect.Lean.Engine.DataFeeds.ApiDataProvider":
+            self._logger.warn(
+                "--data-purchase-limit is ignored because the data provider is not set to download from the QuantConnect API, use --download-data to set that up")
+            return
+
+        lean_config["data-purchase-limit"] = data_purchase_limit
+
     def get_lean_config(self) -> Dict[str, Any]:
         """Reads the Lean config into a dict.
 
