@@ -11,6 +11,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import platform
 import sys
 from typing import Any, List, Optional
 
@@ -28,18 +29,14 @@ class Logger:
     def __init__(self) -> None:
         """Creates a new Logger instance."""
         self._console = Console(markup=False, highlight=False, emoji=False)
-        self._debug_logging_enabled = False
-
-    def enable_debug_logging(self) -> None:
-        """Enables debug messages to be printed."""
-        self._debug_logging_enabled = True
+        self.debug_logging_enabled = False
 
     def debug(self, message: Any) -> None:
         """Logs a debug message if debug logging is enabled.
 
         :param message: the message to log
         """
-        if self._debug_logging_enabled:
+        if self.debug_logging_enabled:
             self._console.print(message, style="grey50")
 
     def info(self, message: Any) -> None:
@@ -63,12 +60,14 @@ class Logger:
         """
         self._console.print(message, style="red")
 
-    def progress(self) -> Progress:
+    def progress(self, prefix: str = "", suffix: str = "{task.percentage:0.0f}%") -> Progress:
         """Creates a Progress instance.
 
+        :param prefix: the text to show before the bar (defaults to a blank string)
+        :param suffix: the text to show after the bar (defaults to the task's percentage)
         :return: a Progress instance which can be used to display progress bars
         """
-        progress = Progress(TextColumn(""), BarColumn(), TextColumn("{task.percentage:0.0f}%"), console=self._console)
+        progress = Progress(TextColumn(prefix), BarColumn(), TextColumn(suffix), console=self._console)
         progress.start()
         return progress
 
@@ -82,6 +81,7 @@ class Logger:
         :return: the chosen option's id
         """
         if len(options) == 1:
+            self.info(f"{text}: {options[0].label}")
             return options[0].id
 
         self.info(f"{text}:")
@@ -112,8 +112,8 @@ class Logger:
         if default is not None:
             text = f"{text} [{'*' * len(default)}]"
 
-        # Masking does not work when the input is not coming from a keyboard
-        if not sys.stdout.isatty():
+        # Masking does not work properly in WSL2 and when the input is not coming from a keyboard
+        if "microsoft" in platform.uname().release.lower() or not sys.stdin.isatty():
             return click.prompt(text, default=default, show_default=False)
 
         while True:
