@@ -14,6 +14,8 @@
 import json
 import os
 import platform
+import shutil
+import site
 import sys
 from datetime import datetime, timezone
 from pathlib import Path
@@ -21,6 +23,7 @@ from typing import List
 
 import pkg_resources
 
+from lean.components.config.lean_config_manager import LeanConfigManager
 from lean.components.config.project_config_manager import ProjectConfigManager
 from lean.components.util.xml_manager import XMLManager
 from lean.models.api import QCLanguage, QCProject
@@ -29,13 +32,18 @@ from lean.models.api import QCLanguage, QCProject
 class ProjectManager:
     """The ProjectManager class provides utilities for handling a single project."""
 
-    def __init__(self, project_config_manager: ProjectConfigManager, xml_manager: XMLManager) -> None:
+    def __init__(self,
+                 project_config_manager: ProjectConfigManager,
+                 lean_config_manager: LeanConfigManager,
+                 xml_manager: XMLManager) -> None:
         """Creates a new ProjectManager instance.
 
         :param project_config_manager: the ProjectConfigManager to use when creating new projects
+        :param lean_config_manager: the LeanConfigManager to get the CLI root directory from
         :param xml_manager: the XMLManager to use when working with XML
         """
         self._project_config_manager = project_config_manager
+        self._lean_config_manager = lean_config_manager
         self._xml_manager = xml_manager
 
     def find_algorithm_file(self, input: Path) -> Path:
@@ -67,7 +75,8 @@ class ProjectManager:
 
         for local_file in local_files:
             posix_path = local_file.as_posix()
-            if "bin/" in posix_path or "obj/" in posix_path or ".ipynb_checkpoints/" in posix_path:
+            if any(f"{part}/" in posix_path for part in
+                   ["bin", "obj", ".ipynb_checkpoints", "backtests", "live", "optimizations"]):
                 continue
 
             files_to_sync.append(local_file)
@@ -114,6 +123,19 @@ class ProjectManager:
         time = round(time.timestamp() * 1e9)
         os.utime(local_file_path, ns=(time, time))
 
+    def copy_code(self, project_dir: Path, output_dir: Path) -> None:
+        """Copies the source code of a project to another directory.
+
+        :param project_dir: the directory of the project
+        :param output_dir: the directory to copy the code to
+        """
+        output_dir.mkdir(parents=True, exist_ok=True)
+
+        for source_file in self.get_files_to_sync(project_dir):
+            target_file = output_dir / source_file.relative_to(project_dir)
+            target_file.parent.mkdir(parents=True, exist_ok=True)
+            shutil.copyfile(source_file, target_file)
+
     def create_new_project(self, project_dir: Path, language: QCLanguage) -> None:
         """Creates a new project directory and fills it with some useful files.
 
@@ -128,6 +150,7 @@ class ProjectManager:
         project_config.set("description", "")
 
         if language == QCLanguage.Python:
+            self._generate_python_library_projects_config()
             self._generate_vscode_python_config(project_dir)
             self._generate_pycharm_config(project_dir)
         else:
@@ -135,6 +158,24 @@ class ProjectManager:
             self._generate_csproj(project_dir)
             self.generate_rider_config()
 
+    def _generate_python_library_projects_config(self) -> None:
+        """Generates the required configuration to enable autocomplete on Python library projects."""
+        try:
+            cli_root_dir = self._lean_config_manager.get_cli_root_directory()
+        except:
+            return
+
+        library_dir = cli_root_dir / "Library"
+        if not library_dir.is_dir():
+            return
+
+        if site.ENABLE_USER_SITE:
+            site_packages_dir = site.getusersitepackages()
+        else:
+            site_packages_dir = site.getsitepackages()[0]
+
+        self._generate_file(Path(site_packages_dir) / "lean-cli.pth", str(library_dir))
+
     def _generate_vscode_python_config(self, project_dir: Path) -> None:
         """Generates Python interpreter configuration and Python debugging configuration for VS Code.
 
@@ -145,28 +186,37 @@ class ProjectManager:
             "python.languageServer": "Pylance"
         }, indent=4))
 
-        self._generate_file(project_dir / ".vscode" / "launch.json", """
-{
-    "version": "0.2.0",
-    "configurations": [
-        {
-            "name": "Debug with Lean CLI",
-            "type": "python",
-            "request": "attach",
-            "connect": {
-                "host": "localhost",
-                "port": 5678
-            },
-            "pathMappings": [
+        launch_config = {
+            "version": "0.2.0",
+            "configurations": [
                 {
-                    "localRoot": "${workspaceFolder}",
-                    "remoteRoot": "/LeanCLI"
+                    "name": "Debug with Lean CLI",
+                    "type": "python",
+                    "request": "attach",
+                    "connect": {
+                        "host": "localhost",
+                        "port": 5678
+                    },
+                    "pathMappings": [
+                        {
+                            "localRoot": "${workspaceFolder}",
+                            "remoteRoot": "/LeanCLI"
+                        }
+                    ]
                 }
             ]
         }
-    ]
-}
-        """)
+
+        try:
+            library_dir = self._lean_config_manager.get_cli_root_directory() / "Library"
+            launch_config["configurations"][0]["pathMappings"].append({
+                "localRoot": str(library_dir),
+                "remoteRoot": "/Library"
+            })
+        except:
+            pass
+
+        self._generate_file(project_dir / ".vscode" / "launch.json", json.dumps(launch_config, indent=4))
 
     def _generate_pycharm_config(self, project_dir: Path) -> None:
         """Generates Python interpreter configuration and Python debugging configuration for PyCharm.
@@ -207,7 +257,14 @@ class ProjectManager:
 </project>
         """)
 
-        self._generate_file(project_dir / ".idea" / "workspace.xml", """
+        try:
+            library_dir = self._lean_config_manager.get_cli_root_directory() / "Library"
+            library_dir = f"$PROJECT_DIR$/{os.path.relpath(library_dir, project_dir)}".replace("\\", "/")
+            library_mapping = f'<mapping local-root="{library_dir}" remote-root="/Library" />'
+        except:
+            library_mapping = ""
+
+        self._generate_file(project_dir / ".idea" / "workspace.xml", f"""
 <?xml version="1.0" encoding="UTF-8"?>
 <project version="4">
   <component name="RunManager" selected="Python Debug Server.Debug with Lean CLI">
@@ -219,6 +276,7 @@ class ProjectManager:
         <option name="pathMappings">
           <list>
             <mapping local-root="$PROJECT_DIR$" remote-root="/LeanCLI" />
+            {library_mapping}
           </list>
         </option>
       </PathMappingSettings>
@@ -336,6 +394,7 @@ class ProjectManager:
         <LangVersion>9</LangVersion>
         <OutputPath>bin/$(Configuration)</OutputPath>
         <AppendTargetFrameworkToOutputPath>false</AppendTargetFrameworkToOutputPath>
+        <DefaultItemExcludes>$(DefaultItemExcludes);backtests/*/code/**;live/*/code/**;optimizations/*/code/**</DefaultItemExcludes>
         <NoWarn>CS0618</NoWarn>
     </PropertyGroup>
     <ItemGroup>
