@@ -25,6 +25,9 @@ CREDENTIALS_CONFIG_PATH = str(Path("~/.lean/credentials").expanduser())
 # The file in which we store when we last checked for updates
 CACHE_PATH = str(Path("~/.lean/cache").expanduser())
 
+# The directory in which modules are stored
+MODULES_DIRECTORY = str(Path("~/.lean/modules").expanduser())
+
 # The default name of the file containing the Lean engine configuration
 DEFAULT_LEAN_CONFIG_FILE_NAME = "lean.json"
 
@@ -61,3 +64,12 @@ UPDATE_CHECK_INTERVAL_DOCKER_IMAGE = 24 * 14
 
 # The interval in hours at which the CLI checks for new announcements
 UPDATE_CHECK_INTERVAL_ANNOUNCEMENTS = 24
+
+# The product id of the Bloomberg module
+BLOOMBERG_PRODUCT_ID = 44
+
+# The product id of the Trading Technologies module
+TRADING_TECHNOLOGIES_PRODUCT_ID = 64
+
+# The product id of the Atreyu module
+ATREYU_PRODUCT_ID = 65
