@@ -18,6 +18,7 @@ from lean.components.api.api_client import APIClient
 from lean.components.cloud.cloud_project_manager import CloudProjectManager
 from lean.components.cloud.cloud_runner import CloudRunner
 from lean.components.cloud.data_downloader import DataDownloader
+from lean.components.cloud.module_manager import ModuleManager
 from lean.components.cloud.pull_manager import PullManager
 from lean.components.cloud.push_manager import PushManager
 from lean.components.config.cli_config_manager import CLIConfigManager
@@ -27,6 +28,7 @@ from lean.components.config.project_config_manager import ProjectConfigManager
 from lean.components.config.storage import Storage
 from lean.components.docker.docker_manager import DockerManager
 from lean.components.docker.lean_runner import LeanRunner
+from lean.components.util.http_client import HTTPClient
 from lean.components.util.logger import Logger
 from lean.components.util.name_generator import NameGenerator
 from lean.components.util.path_manager import PathManager
@@ -47,6 +49,7 @@ class Container(DeclarativeContainer):
     path_manager = Singleton(PathManager)
     temp_manager = Singleton(TempManager)
     xml_manager = Singleton(XMLManager)
+    http_client = Singleton(HTTPClient, logger)
 
     general_storage = Singleton(Storage, file=GENERAL_CONFIG_PATH)
     credentials_storage = Singleton(Storage, file=CREDENTIALS_CONFIG_PATH)
@@ -54,13 +57,14 @@ class Container(DeclarativeContainer):
 
     project_config_manager = Singleton(ProjectConfigManager, xml_manager)
     cli_config_manager = Singleton(CLIConfigManager, general_storage, credentials_storage)
-    lean_config_manager = Singleton(LeanConfigManager, cli_config_manager, project_config_manager)
+    lean_config_manager = Singleton(LeanConfigManager, logger, cli_config_manager, project_config_manager)
     optimizer_config_manager = Singleton(OptimizerConfigManager, logger)
 
-    project_manager = Singleton(ProjectManager, project_config_manager, xml_manager)
+    project_manager = Singleton(ProjectManager, project_config_manager, lean_config_manager, xml_manager)
 
     api_client = Factory(APIClient,
                          logger,
+                         http_client,
                          user_id=cli_config_manager.provided.user_id.get_value()(),
                          api_token=cli_config_manager.provided.api_token.get_value()())
 
@@ -74,6 +78,7 @@ class Container(DeclarativeContainer):
                                       pull_manager,
                                       push_manager,
                                       path_manager)
+    module_manager = Singleton(ModuleManager, logger, api_client, http_client)
 
     docker_manager = Singleton(DockerManager, logger, temp_manager)
 
@@ -82,9 +87,11 @@ class Container(DeclarativeContainer):
                             project_config_manager,
                             lean_config_manager,
                             docker_manager,
-                            temp_manager)
+                            module_manager,
+                            temp_manager,
+                            xml_manager)
 
-    update_manager = Singleton(UpdateManager, logger, cache_storage, docker_manager)
+    update_manager = Singleton(UpdateManager, logger, http_client, cache_storage, docker_manager)
 
 
 container = Container()
