@@ -218,10 +218,10 @@ class QCBacktest(WrappedBaseModel):
             stats.extend(["", ""])
 
         table = Table(box=box.SQUARE)
-        table.add_column("Statistic")
-        table.add_column("Value")
-        table.add_column("Statistic")
-        table.add_column("Value")
+        table.add_column("Statistic", overflow="fold")
+        table.add_column("Value", overflow="fold")
+        table.add_column("Statistic", overflow="fold")
+        table.add_column("Value", overflow="fold")
 
         for i in range(int(len(stats) / 4)):
             start = i * 4
@@ -385,17 +385,6 @@ class QCFullOrganization(WrappedBaseModel):
     data: QCOrganizationData
     members: List[QCOrganizationMember]
 
-    def has_security_master_subscription(self) -> bool:
-        """Returns whether this organization has a Security Master subscription.
-
-        :return: True if the organization has a Security Master subscription, False if not
-        """
-        data_products_product = next((x for x in self.products if x.name == "Data"), None)
-        if data_products_product is None:
-            return False
-
-        return any(x.productId == 37 for x in data_products_product.items)
-
 
 class QCMinimalOrganization(WrappedBaseModel):
     id: str
@@ -498,5 +487,24 @@ class QCDataVendor(WrappedBaseModel):
 
 
 class QCDataInformation(WrappedBaseModel):
+    datasources: Dict[str, Any]
     prices: List[QCDataVendor]
     agreement: str
+
+
+class QCDatasetDelivery(str, Enum):
+    CloudOnly = "cloud only"
+    DownloadOnly = "download only"
+    CloudAndDownload = "cloud & download"
+
+
+class QCDatasetTag(WrappedBaseModel):
+    name: str
+
+
+class QCDataset(WrappedBaseModel):
+    id: int
+    name: str
+    delivery: QCDatasetDelivery
+    vendorName: str
+    tags: List[QCDatasetTag]
